@@ -81,11 +81,10 @@
 #define _WINSOCKAPI_
 #include <wsockcompat.h>
 #include <winsock2.h>
-#undef SOCKLEN_T
-#define SOCKLEN_T unsigned int
+#undef XML_SOCKLEN_T
+#define XML_SOCKLEN_T unsigned int
 #endif
 
-
 /**
  * A couple portability macros
  */
@@ -96,7 +95,7 @@
 #define SOCKET int
 #endif
 #if defined(VMS) || defined(__VMS)
-#define SOCKLEN_T unsigned int
+#define XML_SOCKLEN_T unsigned int
 #endif
 
 #ifdef __BEOS__
@@ -105,6 +104,9 @@
 #endif
 #endif
 
+#ifdef _AIX
+#define ss_family __ss_family
+#endif
 
 #define FTP_COMMAND_OK		200
 #define FTP_SYNTAX_ERROR	500
@@ -260,12 +262,18 @@ xmlNanoFTPCleanup(void) {
 void
 xmlNanoFTPProxy(const char *host, int port, const char *user,
 	        const char *passwd, int type) {
-    if (proxy != NULL)
+    if (proxy != NULL) {
 	xmlFree(proxy);
-    if (proxyUser != NULL)
+	proxy = NULL;
+    }
+    if (proxyUser != NULL) {
 	xmlFree(proxyUser);
-    if (proxyPasswd != NULL)
+	proxyUser = NULL;
+    }
+    if (proxyPasswd != NULL) {
 	xmlFree(proxyPasswd);
+	proxyPasswd = NULL;
+    }
     if (host)
 	proxy = xmlMemStrdup(host);
     if (user)
@@ -288,11 +296,11 @@ xmlNanoFTPProxy(const char *host, int port, const char *user,
 static void
 xmlNanoFTPScanURL(void *ctx, const char *URL) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
-    const char *cur = URL;
-    char buf[XML_NANO_MAX_URLBUF];
-    int indx = 0;
-    int port = 0;
+    xmlURIPtr uri;
 
+    /*
+     * Clear any existing data from the context
+     */
     if (ctxt->protocol != NULL) { 
         xmlFree(ctxt->protocol);
 	ctxt->protocol = NULL;
@@ -306,122 +314,38 @@ xmlNanoFTPScanURL(void *ctx, const char *URL) {
 	ctxt->path = NULL;
     }
     if (URL == NULL) return;
-    buf[indx] = 0;
-    while ((*cur != 0) && (indx < XML_NANO_MAX_URLBUF - 1)) {
-        if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
-	    buf[indx] = 0;
-	    ctxt->protocol = xmlMemStrdup(buf);
-	    indx = 0;
-            cur += 3;
-	    break;
-	}
-	buf[indx++] = *cur++;
-    }
-    if (*cur == 0) return;
 
-    buf[indx] = 0;
-    /* allow user@ and user:pass@ forms */
-    {
-	const char *p = strchr(cur, '@');
-	if(p) {
-	    while(indx < XML_NANO_MAX_URLBUF-1) {
-		if(cur[0] == ':' || cur[0] == '@') break;
-		buf[indx++] = *cur++;
-	    }
-	    buf[indx] = 0;
-	    ctxt->user = xmlMemStrdup(buf);
-	    indx = 0;
-	    if(cur[0] == ':') {
-		cur++;
-		while(indx < XML_NANO_MAX_URLBUF-1) {
-		    if(cur[0] == '@') break;
-		    buf[indx++] = *cur++;
-		}
-		buf[indx] = 0;
-		ctxt->passwd = xmlMemStrdup(buf);
-		indx = 0;
-	    }
-	    cur = p+1;
+    uri = xmlParseURI(URL);
+    if (uri == NULL)
+	return;
+
+    if ((uri->scheme == NULL) || (uri->server == NULL)) {
+	xmlFreeURI(uri);
+	return;
+    }
+    
+    ctxt->protocol = xmlMemStrdup(uri->scheme);
+    ctxt->hostname = xmlMemStrdup(uri->server);
+    if (uri->path != NULL)
+	ctxt->path = xmlMemStrdup(uri->path);
+    else
+	ctxt->path = xmlMemStrdup("/");
+    if (uri->port != 0)
+	ctxt->port = uri->port;
+
+    if (uri->user != NULL) {
+	char *cptr;
+	if ((cptr=strchr(uri->user, ':')) == NULL)
+	    ctxt->user = xmlMemStrdup(uri->user);
+	else {
+	    ctxt->user = (char *)xmlStrndup((xmlChar *)uri->user,
+			    (cptr - uri->user));
+	    ctxt->passwd = xmlMemStrdup(cptr+1);
 	}
     }
 
-    while (indx < XML_NANO_MAX_URLBUF - 1) {
-	if ((strchr (cur, '[') && !strchr (cur, ']')) ||
-		(!strchr (cur, '[') && strchr (cur, ']'))) {
-	    xmlGenericError (xmlGenericErrorContext, "\nxmlNanoFTPScanURL: %s",
-		    "Syntax Error\n");
-	    return;
-	}
+    xmlFreeURI(uri);
 
-	if (cur[0] == '[') {
-	    cur++;
-	    while ((cur[0] != ']') && (indx < XML_NANO_MAX_URLBUF-1))
-		buf[indx++] = *cur++;
-	    if (indx >= XML_NANO_MAX_URLBUF-1) {
-		xmlGenericError(xmlGenericErrorContext,
-		                "\nxmlNanoFTPScanURL: %s", "Syntax Error\n");
-		return;
-	    }
-
-	    if (!strchr (buf, ':')) {
-		xmlGenericError (xmlGenericErrorContext, "\nxmlNanoFTPScanURL: %s",
-			"Use [IPv6]/IPv4 format\n");
-		return;
-	    }
-
-	    buf[indx] = 0;
-	    ctxt->hostname = xmlMemStrdup (buf);
-	    indx = 0;
-	    cur += 1;
-	    if (cur[0] == ':') {
-		cur++;
-		while (*cur >= '0' && *cur <= '9') {
-		    port *= 10;
-		    port += *cur - '0';
-		    cur++;
-		}
-
-		if (port != 0) ctxt->port = port;
-		while ((cur[0] != '/') && (*cur != 0))
-		    cur++;
-	    }
-	    break;
-	}
-	else {     /* address is an IPv4 one*/
-	    if (cur[0] == ':') {
-		buf[indx] = 0;
-		ctxt->hostname = xmlMemStrdup (buf);
-		indx = 0;
-		cur += 1;
-		while ((*cur >= '0') && (*cur <= '9')) {
-		    port *= 10;
-		    port += *cur - '0';
-		    cur++;
-		}
-		if (port != 0) ctxt->port = port;
-		while ((cur[0] != '/') && (*cur != 0)) 
-		    cur++;
-		break;
-	    }
-	    if ((*cur == '/') || (*cur == 0)) {
-		buf[indx] = 0;
-		ctxt->hostname = xmlMemStrdup (buf);
-		indx = 0;
-		break;
-	    }
-	}
-	buf[indx++] = *cur++;
-    }
-    if (*cur == 0) 
-        ctxt->path = xmlMemStrdup("/");
-    else {
-        indx = 0;
-        buf[indx] = 0;
-	while ((*cur != 0) && (indx < XML_NANO_MAX_URLBUF-1))
-	    buf[indx++] = *cur++;
-	buf[indx] = 0;
-	ctxt->path = xmlMemStrdup(buf);
-    }	
 }
 
 /**
@@ -441,10 +365,7 @@ xmlNanoFTPScanURL(void *ctx, const char *URL) {
 int
 xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
-    const char *cur = URL;
-    char buf[XML_NANO_MAX_URLBUF];
-    int indx = 0;
-    int port = 0;
+    xmlURIPtr uri;
 
     if (URL == NULL)
 	return(-1);
@@ -454,104 +375,37 @@ xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
 	return(-1);
     if (ctxt->hostname == NULL)
 	return(-1);
-    buf[indx] = 0;
-    while ((*cur != 0) && (indx < XML_NANO_MAX_URLBUF-1)) {
-        if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
-	    buf[indx] = 0;
-	    if (strcmp(ctxt->protocol, buf))
-		return(-1);
-	    indx = 0;
-            cur += 3;
-	    break;
-	}
-	buf[indx++] = *cur++;
-    }
-    if (*cur == 0)
+
+    uri = xmlParseURI(URL);
+    if (uri == NULL)
 	return(-1);
 
-    buf[indx] = 0;
-    while (indx < XML_NANO_MAX_URLBUF-1) {
-	if ((strchr (cur, '[') && !strchr (cur, ']')) ||
-		(!strchr (cur, '[') && strchr (cur, ']'))) {
-	    xmlGenericError (xmlGenericErrorContext, "\nxmlNanoFTPUpdateURL: %s",
-		    "Syntax Error\n");
-	    return (-1);
-	}
-
-	if (cur[0] == '[') {
-	    cur++;
-	    while ((cur[0] != ']') && (indx < XML_NANO_MAX_URLBUF-1))
-		buf[indx++] = *cur++;
-
-	    if (!strchr (buf, ':')) {
-		xmlGenericError (xmlGenericErrorContext, "\nxmlNanoFTPUpdateURL: %s",
-			"Use [IPv6]/IPv4 format\n");
-		return (-1);
-	    }
-
-	    buf[indx] = 0;
-	    if (strcmp (ctxt->hostname, buf))
-		return (-1);
-	    indx = 0;
-	    cur += 1;
-	    if (cur[0] == ':') {
-		cur++;
-		while (*cur >= '0' && *cur <= '9') {
-		    port *= 10;
-		    port += *cur - '0';
-		    cur++;
-		}
-
-		if (port != ctxt->port)
-		    return (-1);
-		while ((cur[0] != '/') && (*cur != 0))
-		    cur++;
-	    }
-	    break;
-	}
-	else {
-	    if (cur[0] == ':') {
-		buf[indx] = 0;
-		if (strcmp (ctxt->hostname, buf))
-		    return (-1);
-		indx = 0;
-		cur += 1;
-		while ((*cur >= '0') && (*cur <= '9')) {
-		    port *= 10;
-		    port += *cur - '0';
-		    cur++;
-		}
-		if (port != ctxt->port)
-		    return (-1);
-		while ((cur[0] != '/') && (*cur != 0)) 
-		    cur++;
-		break;
-	    }
-	    if ((*cur == '/') || (*cur == 0)) {
-		buf[indx] = 0;
-		if (strcmp (ctxt->hostname, buf))
-		    return (-1);
-		indx = 0;
-		break;
-	    }
-	}
-	buf[indx++] = *cur++;
+    if ((uri->scheme == NULL) || (uri->server == NULL)) {
+	xmlFreeURI(uri);
+	return(-1);
     }
+    if ((strcmp(ctxt->protocol, uri->scheme)) ||
+	(strcmp(ctxt->hostname, uri->server)) ||
+	((uri->port != 0) && (ctxt->port != uri->port))) {
+	xmlFreeURI(uri);
+	return(-1);
+    }
+
+    if (uri->port != 0)
+	ctxt->port = uri->port;
+
     if (ctxt->path != NULL) {
 	xmlFree(ctxt->path);
 	ctxt->path = NULL;
     }
 
-    if (*cur == 0) 
+    if (uri->path == NULL) 
         ctxt->path = xmlMemStrdup("/");
-    else {
-        indx = 0;
-        buf[indx] = 0;
-	while ((*cur != 0) && (indx < XML_NANO_MAX_URLBUF-1))
-	    buf[indx++] = *cur++;
-	buf[indx] = 0;
-	ctxt->path = xmlMemStrdup(buf);
-    }	
+    else
+	ctxt->path = xmlMemStrdup(uri->path);
+
+    xmlFreeURI(uri);
+
     return(0);
 }
 
@@ -567,105 +421,38 @@ xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
 
 void
 xmlNanoFTPScanProxy(const char *URL) {
-    const char *cur = URL;
-    char buf[XML_NANO_MAX_URLBUF];
-    int indx = 0;
-    int port = 0;
+    xmlURIPtr uri;
 
     if (proxy != NULL) { 
         xmlFree(proxy);
 	proxy = NULL;
     }
-    if (proxyPort != 0) { 
-	proxyPort = 0;
-    }
+    proxyPort = 0;
+
 #ifdef DEBUG_FTP
     if (URL == NULL)
-	xmlGenericError(xmlGenericErrorContext, "Removing FTP proxy info\n");
+	xmlGenericError(xmlGenericErrorContext,
+		"Removing FTP proxy info\n");
     else
-	xmlGenericError(xmlGenericErrorContext, "Using FTP proxy %s\n", URL);
+	xmlGenericError(xmlGenericErrorContext,
+		"Using FTP proxy %s\n", URL);
 #endif
     if (URL == NULL) return;
-    buf[indx] = 0;
-    while ((*cur != 0) && (indx < XML_NANO_MAX_URLBUF-1)) {
-        if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
-	    buf[indx] = 0;
-	    indx = 0;
-            cur += 3;
-	    break;
-	}
-	buf[indx++] = *cur++;
+
+    uri = xmlParseURI(URL);
+    if ((uri == NULL) || (uri->scheme == NULL) ||
+	(strcmp(uri->scheme, "ftp")) || (uri->server == NULL)) {
+	__xmlIOErr(XML_FROM_FTP, XML_FTP_URL_SYNTAX, "Syntax Error\n");
+	if (uri != NULL)
+	    xmlFreeURI(uri);
+	return;
     }
-    if (*cur == 0) return;
+    
+    proxy = xmlMemStrdup(uri->server);
+    if (uri->port != 0)
+	proxyPort = uri->port;
 
-    buf[indx] = 0;
-    while (indx < XML_NANO_MAX_URLBUF-1) {
-	if ((strchr (cur, '[') && !strchr (cur, ']')) ||
-		(!strchr (cur, '[') && strchr (cur, ']'))) {
-	    xmlGenericError (xmlGenericErrorContext, "\nxmlNanoFTPScanProxy: %s",
-		    "Syntax error\n");
-	    return;
-	}
-
-	if (cur[0] == '[') {
-	    cur++;
-	    while ((cur[0] != ']') && (indx < XML_NANO_MAX_URLBUF-1))
-		buf[indx++] = *cur++;
-            if (indx >= XML_NANO_MAX_URLBUF-1) {
-		xmlGenericError (xmlGenericErrorContext,
-			  "\nxmlNanoFTPScanProxy: %s", "Syntax error\n");
-		return;
-	    }
-
-	    if (!strchr (buf, ':')) {
-		xmlGenericError (xmlGenericErrorContext, "\nxmlNanoFTPScanProxy: %s",
-			"Use [IPv6]/IPv4 format\n");
-		return;
-	    }
-
-	    buf[indx] = 0;
-	    proxy = xmlMemStrdup (buf);
-	    indx = 0;
-	    cur += 1;
-	    if (cur[0] == ':') {
-		cur++;
-		while (*cur >= '0' && *cur <= '9') {
-		    port *= 10;
-		    port += *cur - '0';
-		    cur++;
-		}
-
-		if (port != 0) proxyPort = port;
-		while ((cur[0] != '/') && (*cur != 0))
-		    cur++;
-	    }
-	    break;
-	}
-	else {
-	    if (cur[0] == ':') {
-		buf[indx] = 0;
-		proxy = xmlMemStrdup (buf);
-		indx = 0;
-		cur += 1;
-		while ((*cur >= '0') && (*cur <= '9')) {
-		    port *= 10;
-		    port += *cur - '0';
-		    cur++;
-		}
-		if (port != 0) proxyPort = port;
-		while ((cur[0] != '/') && (*cur != 0)) 
-		    cur++;
-		break;
-	    }
-	    if ((*cur == '/') || (*cur == 0)) {
-		buf[indx] = 0;
-		proxy = xmlMemStrdup (buf);
-		indx = 0;
-		break;
-	    }
-	}
-	buf[indx++] = *cur++;
-    }
+    xmlFreeURI(uri);
 }
 
 /**
@@ -697,11 +484,11 @@ xmlNanoFTPNewCtxt(const char *URL) {
     ret->controlFd = -1;
 
     unescaped = xmlURIUnescapeString(URL, 0, NULL);
-    if (unescaped != NULL)
+    if (unescaped != NULL) {
 	xmlNanoFTPScanURL(ret, unescaped);
-    else if (URL != NULL)
+	xmlFree(unescaped);
+    } else if (URL != NULL)
 	xmlNanoFTPScanURL(ret, URL);
-    xmlFree(unescaped);
 
     return(ret);
 }
@@ -777,6 +564,8 @@ xmlNanoFTPGetMore(void *ctx) {
     int len;
     int size;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0)) return(-1);
+
     if ((ctxt->controlBufIndex < 0) || (ctxt->controlBufIndex > FTP_BUF_SIZE)) {
 #ifdef DEBUG_FTP
         xmlGenericError(xmlGenericErrorContext,
@@ -856,6 +645,8 @@ xmlNanoFTPReadResponse(void *ctx) {
     int len;
     int res = -1, cur = -1;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0)) return(-1);
+
 get_more:
     /*
      * Assumes everything up to controlBuf[controlBufIndex] has been read
@@ -943,6 +734,7 @@ xmlNanoFTPCheckResponse(void *ctx) {
     fd_set rfd;
     struct timeval tv;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0)) return(-1);
     tv.tv_sec = 0;
     tv.tv_usec = 0;
     FD_ZERO(&rfd);
@@ -1031,6 +823,8 @@ xmlNanoFTPQuit(void *ctx) {
     char buf[200];
     int len, res;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0)) return(-1);
+
     snprintf(buf, sizeof(buf), "QUIT\r\n");
     len = strlen(buf);
 #ifdef DEBUG_FTP
@@ -1442,6 +1236,8 @@ xmlNanoFTPConnectTo(const char *server, int port) {
     xmlNanoFTPInit();
     if (server == NULL) 
 	return(NULL);
+    if (port <= 0)
+	return(NULL);
     ctxt = (xmlNanoFTPCtxtPtr) xmlNanoFTPNewCtxt(NULL);
     ctxt->hostname = xmlMemStrdup(server);
     if (port != 0)
@@ -1465,12 +1261,15 @@ xmlNanoFTPConnectTo(const char *server, int port) {
  */
 
 int
-xmlNanoFTPCwd(void *ctx, char *directory) {
+xmlNanoFTPCwd(void *ctx, const char *directory) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     char buf[400];
     int len;
     int res;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0)) return(-1);
+    if (directory == NULL) return 0;
+
     /*
      * Expected response code for CWD:
      *
@@ -1511,12 +1310,15 @@ xmlNanoFTPCwd(void *ctx, char *directory) {
  */
 
 int
-xmlNanoFTPDele(void *ctx, char *file) {
+xmlNanoFTPDele(void *ctx, const char *file) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     char buf[400];
     int len;
     int res;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0) || (file == NULL)) return(-1);
+    if (file == NULL) return (0);
+
     /*
      * Expected response code for DELE:
      *
@@ -1570,7 +1372,9 @@ xmlNanoFTPGetConnection(void *ctx) {
 #else
     struct sockaddr_in dataAddr;
 #endif
-    SOCKLEN_T dataAddrLen;
+    XML_SOCKLEN_T dataAddrLen;
+
+    if (ctxt == NULL) return(-1);
 
     memset (&dataAddr, 0, sizeof(dataAddr));
 #ifdef SUPPORT_IP6
@@ -1735,6 +1539,8 @@ xmlNanoFTPCloseConnection(void *ctx) {
     fd_set rfd, efd;
     struct timeval tv;
 
+    if ((ctxt == NULL) || (ctxt->controlFd < 0)) return(-1);
+
     closesocket(ctxt->dataFd); ctxt->dataFd = -1;
     tv.tv_sec = 15;
     tv.tv_usec = 0;
@@ -1907,7 +1713,7 @@ xmlNanoFTPParseList(const char *list, ftpListCallback callback, void *userData)
 
 int
 xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
-	       char *filename) {
+	       const char *filename) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     char buf[4096 + 1];
     int len, res;
@@ -1915,6 +1721,7 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
     fd_set rfd, efd;
     struct timeval tv;
 
+    if (ctxt == NULL) return (-1);
     if (filename == NULL) {
         if (xmlNanoFTPCwd(ctxt, ctxt->path) < 1)
 	    return(-1);
@@ -2019,6 +1826,8 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     char buf[300];
     int res, len;
+    if (ctx == NULL)
+	return(-1);
     if ((filename == NULL) && (ctxt->path == NULL))
 	return(-1);
     ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
@@ -2086,6 +1895,7 @@ xmlNanoFTPGet(void *ctx, ftpDataCallback callback, void *userData,
     fd_set rfd;
     struct timeval tv;
 
+    if (ctxt == NULL) return(-1);
     if ((filename == NULL) && (ctxt->path == NULL))
 	return(-1);
     if (callback == NULL)
@@ -2155,7 +1965,8 @@ xmlNanoFTPRead(void *ctx, void *dest, int len) {
 
     len = recv(ctxt->dataFd, dest, len, 0);
     if (len <= 0) {
-        __xmlIOErr(XML_FROM_FTP, 0, "recv failed");
+	if (len < 0)
+	    __xmlIOErr(XML_FROM_FTP, 0, "recv failed");
 	xmlNanoFTPCloseConnection(ctxt);
     }
 #ifdef DEBUG_FTP
@@ -2293,3 +2104,5 @@ int main(int argc, char **argv) {
 }
 #endif /* STANDALONE */
 #endif /* LIBXML_FTP_ENABLED */
+#define bottom_nanoftp
+#include "elfgcchack.h"
