@@ -1082,7 +1082,7 @@ xmlCompileStepPattern(xmlPatParserContextPtr ctxt) {
 		ctxt->error = 1;
 		goto error;
 	    }
-	    xmlFree(name);
+	    /* NOT REACHED xmlFree(name); */
 	}
     } else if (CUR == '*') {
         if (name != NULL) {
@@ -2075,4 +2075,6 @@ xmlPatternFromRoot(xmlPatternPtr comp) {
 
 }
 
+#define bottom_pattern
+#include "elfgcchack.h"
 #endif /* LIBXML_PATTERN_ENABLED */
