@@ -451,7 +451,7 @@ xmlXIncludeParseFile(xmlXIncludeCtxtPtr ctxt, const char *URL) {
     if ((pctxt->directory == NULL) && (directory != NULL))
         pctxt->directory = (char *) xmlStrdup((xmlChar *) directory);
 
-    pctxt->loadsubset = XML_DETECT_IDS;
+    pctxt->loadsubset |= XML_DETECT_IDS;
 
     xmlParseDocument(pctxt);
 
@@ -2468,3 +2468,5 @@ xmlXIncludeProcessNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr node) {
 
 #else /* !LIBXML_XINCLUDE_ENABLED */
 #endif
+#define bottom_xinclude
+#include "elfgcchack.h"
