@@ -31,8 +31,8 @@
 #define _WINSOCKAPI_
 #include <wsockcompat.h>
 #include <winsock2.h>
-#undef SOCKLEN_T
-#define SOCKLEN_T unsigned int
+#undef XML_SOCKLEN_T
+#define XML_SOCKLEN_T unsigned int
 #endif
 
 #ifdef HAVE_SYS_TIMEB_H
@@ -695,6 +695,7 @@ xmlHTMLValidityError(void *ctx, const char *msg, ...)
 
     xmlHTMLPrintFileContext(input);
     xmlHTMLEncodeSend();
+    progresult = XMLLINT_ERR_VALID;
 }
 
 /**
